use axum::{
    async_trait,
    extract::FromRequestParts,
    http::request::Parts,
    response::{IntoResponse, Redirect, Response},
};
use tower_sessions::Session;

pub trait LoggedIn {
    fn logged_in(&self) -> bool;
}

impl LoggedIn for Session {
    fn logged_in(self: &Session) -> bool {
        self.get::<bool>("login")
            .unwrap_or(Some(false))
            .unwrap_or(false)
    }
}

// implement extractor that checks we're logged in or redirect to login page
// for common case
pub struct AuthRedirect;

impl IntoResponse for AuthRedirect {
    fn into_response(self) -> Response {
        Redirect::temporary("/").into_response()
    }
}

pub struct CheckAuth();

#[async_trait]
impl<S> FromRequestParts<S> for CheckAuth
where
    S: Send + Sync,
{
    type Rejection = AuthRedirect;

    async fn from_request_parts(req: &mut Parts, state: &S) -> Result<Self, Self::Rejection> {
        // Allow everything in debug mode
        if cfg!(debug_assertions) {
            return Ok(CheckAuth());
        }

        let Ok(session) = Session::from_request_parts(req, state).await else {
            return Err(AuthRedirect);
        };

        if !session.logged_in() {
            return Err(AuthRedirect);
        }

        Ok(CheckAuth())
    }
}
