// SPDX-License-Identifier: MIT

// re-export non-conditional code
pub use inner::bind;

#[cfg(any(feature = "tls", feature = "rustls"))]
pub use inner::{TlsAcceptor, TlsStream};

#[cfg(not(any(feature = "tls", feature = "rustls")))]
mod inner {
    use anyhow::{Error, Result};
    use axum_server::Server;
    use std::net::SocketAddr;

    pub async fn bind(_addr: SocketAddr) -> Result<Server> {
        Err(Error::msg("not compiled with tls support"))
    }
}

// prefer rustls over (openssl) tls if both are set.
// This should never happen so just pick either.
#[cfg(all(feature = "tls", not(feature = "rustls")))]
mod inner {
    use anyhow::{Context, Result};
    use axum_server::tls_openssl::{OpenSSLAcceptor, OpenSSLConfig};
    use axum_server::Server;
    use hyper::server::conn::AddrStream;
    use std::net::SocketAddr;

    use crate::args::args;
    use crate::server::dual::bind_dual_protocol;
    use crate::server::dual::DualProtocolAcceptor;

    fn tls_config() -> Result<OpenSSLConfig> {
        OpenSSLConfig::from_pem_file(&args().tls_cert, &args().tls_key)
            .context("Could not initialize ssl context")
    }

    pub type TlsAcceptor = OpenSSLAcceptor;
    pub type TlsStream = tokio_openssl::SslStream<AddrStream>;

    pub async fn bind(addr: SocketAddr) -> Result<Server<DualProtocolAcceptor>> {
        let config = tls_config()?;
        let tls_acceptor = OpenSSLAcceptor::new(config);
        Ok(bind_dual_protocol(addr, tls_acceptor))
    }
}

#[cfg(feature = "rustls")]
mod inner {
    use anyhow::{Context, Result};
    use axum_server::tls_rustls::{RustlsAcceptor, RustlsConfig};
    use axum_server::Server;
    use hyper::server::conn::AddrStream;
    use std::net::SocketAddr;

    use crate::args::args;
    use crate::server::dual::bind_dual_protocol;
    use crate::server::dual::DualProtocolAcceptor;

    async fn tls_config() -> Result<RustlsConfig> {
        RustlsConfig::from_pem_file(&args().tls_cert, &args().tls_key)
            .await
            .context("Could not initialize ssl context")
    }

    pub type TlsAcceptor = RustlsAcceptor;
    pub type TlsStream = tokio_rustls::server::TlsStream<AddrStream>;

    pub async fn bind(addr: SocketAddr) -> Result<Server<DualProtocolAcceptor>> {
        let config = tls_config().await?;
        let tls_acceptor = RustlsAcceptor::new(config);
        Ok(bind_dual_protocol(addr, tls_acceptor))
    }
}
