// SPDX-License-Identifier: MIT

use askama::Template;
use axum::{
    extract::Form,
    response::{IntoResponse, Redirect},
    routing::{get, post},
    Router,
};
use serde::Deserialize;
use std::collections::HashMap;

use crate::common::{self, html_template::HtmlTemplate, CheckAuth, Config};
use crate::error::PageResult;

#[derive(Deserialize)]
struct WwanParam {
    apn: String,
    username: String,
    password: String,
    auth_type: String,
    device: String,
    mccmnc: String,
}

#[derive(Deserialize, Debug)]
struct WwanSaved {
    con: String,
    apn: String,
    user: String,
}

#[derive(Template)]
#[template(path = "../src/wwan/templates/wwan.html")]
struct WwanTemplate {
    wwan_info: HashMap<&'static str, String>,
    wwan_addr: HashMap<&'static str, String>,
    wwan_saved: Vec<WwanSaved>,
    has_wwan: bool,
}

pub trait WwanRoutes {
    fn wwan_routes(self) -> Self;
}

impl WwanRoutes for Router {
    fn wwan_routes(self) -> Self {
        self.route("/wwan", get(wwan))
            .route("/wwan_setup", post(wwan_setup))
            .route("/wwan_delete", post(wwan_delete))
            .route("/wwan_connect", post(wwan_connect))
            .route("/wwan_disconnect", post(wwan_disconnect))
    }
}

async fn wwan(_loggedin: CheckAuth) -> PageResult {
    let interface = common::wwan_interface().unwrap_or("--");
    let wwan_addr = common::current_address("ttyCommModem", interface).await?;
    let wwan_info = common::wwan_info(common::WWAN_CON_NAME).await?;

    let args = &["get_saved_wwan.sh"];
    let output = common::exec_command(args).await?;
    let stdout = String::from_utf8_lossy(&output.stdout);
    let wwan_saved = stdout
        .lines()
        .map(|s| s.trim().split(';').collect::<Vec<&str>>())
        .map(|a| WwanSaved {
            con: a.first().unwrap_or(&"--").to_string(),
            apn: a.get(1).unwrap_or(&"--").to_string(),
            user: a.get(2).unwrap_or(&"--").to_string(),
        })
        .collect();

    let template = WwanTemplate {
        wwan_addr,
        wwan_info,
        wwan_saved,
        has_wwan: Config::get().hardware.has_wwan,
    };
    Ok(HtmlTemplate::new("WWAN設定", template).into_response())
}

async fn wwan_setup(_loggedin: CheckAuth, Form(wwan_param): Form<WwanParam>) -> PageResult {
    let mut args = vec![
        "wwan_setup.sh",
        common::WWAN_CON_NAME,
        &wwan_param.device,
        &wwan_param.apn,
        &wwan_param.username,
        &wwan_param.password,
        &wwan_param.auth_type,
    ];

    if !wwan_param.mccmnc.is_empty() {
        args.push("-m");
        args.push(&wwan_param.mccmnc);
    }

    common::exec_command(&args).await?;

    Ok(Redirect::to("/wwan").into_response())
}

async fn wwan_delete(_loggedin: CheckAuth) -> PageResult {
    let args = &["wwan_delete.sh", common::WWAN_CON_NAME];
    common::exec_command(args).await?;

    Ok(Redirect::to("/wwan").into_response())
}

async fn wwan_connect(_loggedin: CheckAuth) -> PageResult {
    let args = &["wwan_connect.sh", common::WWAN_CON_NAME, "up"];
    common::exec_command(args).await?;

    Ok(Redirect::to("/wwan").into_response())
}

async fn wwan_disconnect(_loggedin: CheckAuth) -> PageResult {
    let args = &["wwan_connect.sh", common::WWAN_CON_NAME, "down"];
    common::exec_command(args).await?;

    Ok(Redirect::to("/wwan").into_response())
}
